
/*
   The 4xGLSoft shader - Bloom II
   
   Copyright (C) 2009 guest(r) - guest.r@gmail.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/


uniform sampler2D screenTexture;
uniform vec2 textureSize;
uniform vec2 usedTextureSize;

float hd(float x)
{ 
  float y = x*1.1547-1.0;
  float z = x*0.8660254;
  y = (sin(y*1.5707)+1.0)*0.866025;
  return ((1.0-z)*x+z*y);
}

void main()
{
    float x = 0.5 / textureSize.x;
    float y = 0.5 / textureSize.y;
    vec2 dg1 = vec2( x,y);  vec2 dg2 = vec2(-x,y);
    vec2 sd1 = dg1*0.5;     vec2 sd2 = dg2*0.5;
    vec2 ddx = vec2(x,0.0); vec2 ddy = vec2(0.0,y);

    vec4 v_vTexcoord[7];

    v_vTexcoord[0] = gl_TexCoord[0];
    v_vTexcoord[1].xy = gl_TexCoord[0].xy - sd1;
    v_vTexcoord[2].xy = gl_TexCoord[0].xy - sd2;
    v_vTexcoord[3].xy = gl_TexCoord[0].xy + sd1;
    v_vTexcoord[4].xy = gl_TexCoord[0].xy + sd2;
    v_vTexcoord[5].xy = gl_TexCoord[0].xy - dg1;
    v_vTexcoord[6].xy = gl_TexCoord[0].xy + dg1;
    v_vTexcoord[5].zw = gl_TexCoord[0].xy - dg2;
    v_vTexcoord[6].zw = gl_TexCoord[0].xy + dg2;
    v_vTexcoord[1].zw = gl_TexCoord[0].xy - ddy;
    v_vTexcoord[2].zw = gl_TexCoord[0].xy + ddx;
    v_vTexcoord[3].zw = gl_TexCoord[0].xy + ddy;
    v_vTexcoord[4].zw = gl_TexCoord[0].xy - ddx;


    vec3 c11 = texture2D(screenTexture, v_vTexcoord[0].xy).xyz;
    vec3 s00 = texture2D(screenTexture, v_vTexcoord[1].xy).xyz;
    vec3 s20 = texture2D(screenTexture, v_vTexcoord[2].xy).xyz;
    vec3 s22 = texture2D(screenTexture, v_vTexcoord[3].xy).xyz;
    vec3 s02 = texture2D(screenTexture, v_vTexcoord[4].xy).xyz;
    vec3 c00 = texture2D(screenTexture, v_vTexcoord[5].xy).xyz;
    vec3 c22 = texture2D(screenTexture, v_vTexcoord[6].xy).xyz;
    vec3 c20 = texture2D(screenTexture, v_vTexcoord[5].zw).xyz;
    vec3 c02 = texture2D(screenTexture, v_vTexcoord[6].zw).xyz;
    vec3 c01 = texture2D(screenTexture, v_vTexcoord[1].zw).xyz;
    vec3 c21 = texture2D(screenTexture, v_vTexcoord[2].zw).xyz;
    vec3 c10 = texture2D(screenTexture, v_vTexcoord[3].zw).xyz;
    vec3 c12 = texture2D(screenTexture, v_vTexcoord[4].zw).xyz;
    vec3 dt = vec3(1.0,1.0,1.0);

    float d1=dot(abs(c00-c22),dt)+0.001;
    float d2=dot(abs(c20-c02),dt)+0.001;
    float hl=dot(abs(c01-c21),dt)+0.001;
    float vl=dot(abs(c10-c12),dt)+0.001;
    float m1=dot(abs(s00-s22),dt)+0.001;
    float m2=dot(abs(s02-s20),dt)+0.001;

    vec3 t1=(hl*(c10+c12)+vl*(c01+c21)+(hl+vl)*c11)/(3.0*(hl+vl));
    vec3 t2=(d1*(c20+c02)+d2*(c00+c22)+(d1+d2)*c11)/(3.0*(d1+d2));
    
    c11 =.25*(t1+t2+(m2*(s00+s22)+m1*(s02+s20))/(m1+m2));

    d1 = length(c11);
    c11= vec3(pow(c11,vec3(1.2)));

        gl_FragColor.xyz=hd(d1)*normalize(c11);
}
